require 'spec_helper'
require 'aha'

describe Hash do
  before(:each) do
     @data = {:name     => 'Oculus VR',
              :founders => ['Palmer Luckey', 'Brendan Iribe', 'Michael Antonov', 'Nate Mitchell', 'Jack McCauley'],
              :ceo      => {'name' => 'Brendan Iribe'},
              'cto'     => {:name   => 'John Carmack',
                            'age'   => 44,
                            :spouse => {:name => 'Katherine Anna Kang'}}}
  end

  it 'should destructure nested hashes' do
    company_name, ceo_name, cto_name, cto_age, cto_spouse_name =
      @data.vals_at(:name, [:ceo, 'name'], ['cto', :name, 'age', [:spouse, :name]])

    company_name.should == 'Oculus VR'
    ceo_name.should == 'Brendan Iribe'
    cto_name.should == 'John Carmack'
    cto_age.should == 44
    cto_spouse_name.should == 'Katherine Anna Kang'
  end

  it 'should retrieve nested values' do
    @data.get_in('cto', :spouse, :name).should == 'Katherine Anna Kang'
  end

  it 'should set nested values' do
    @data.put_in! 'cto', :name, 'Alan Kay'
    @data['cto'][:name].should == 'Alan Kay'
  end

  it 'should updated nested values' do
    @data.update_in!('cto', 'age') { |age| age + 5 }
    @data['cto']['age'].should == 49
  end

  it 'should delete nested values' do
    @data.delete_in! 'cto', :spouse
    @data['cto'][:spouse].should be_nil
  end

  it 'should exclude keys in nested hashes' do
    @data.exclude! :founders, :ceo, ['cto', 'age', :spouse]
    @data.should == {:name => 'Oculus VR', 'cto' => {:name => 'John Carmack'}}
  end

  it 'should return a new hash from given keys' do
    @data.only(:name, ['cto', [:spouse, :name]]). should ==
      {:name => 'Oculus VR',
       'cto' => {:spouse => {:name => 'Katherine Anna Kang'}}}
  end
end
